@@ -8,6 +8,7 @@ class PCM_source;
 class RprTake {
 public:
     RprTake(MediaItem_Take *take);
+    operator bool() const { return !!mTake; }
 
     double getPlayRate() const;
     void setPlayRate(double playRate);
@@ -27,8 +28,6 @@ public:
     void setName(const char *name);
     const char *getName();
 
-    static RprTake createFromMidiEditor();
-
     /* Conversion */
     MediaItem_Take *toReaper() const;
 
